import argparse
from functools import cached_property

from ..__meta__ import DESCRIPTION, PROGRAMM
from ..download import Downloader
from ..utils import BColors
from ..utils.string_processing import unify
from .utils import HELP_MESSAGE, ask_for_bool, ask_for_create


class DevelopmentCli:
    def __init__(self, args: argparse.Namespace):
        self.args = args

        if args.genre:
            self.genre = args.genre

        self.downloader: Downloader = Downloader()

    @cached_property
    def genre(self) -> str:
        """This is a cached property, which means if it isn't set in the constructor or before it is accessed,
        the program will be thrown in a shell

        Returns:
            str: the genre that should be used
        """
        option_string = f"{BColors.HEADER}Genres{BColors.ENDC}"
        genre_map = {}

        _string_list = []
        for i, genre in enumerate(self.downloader.get_existing_genres()):
            option_string += f"\n{BColors.BOLD}{i}{BColors.ENDC}: {genre}"

            genre_map[str(i)] = genre
            genre_map[unify(genre)] = genre

        genre = None
        while genre is None:
            print(option_string)
            print()

            i = input("> ") 
            u = unify(i)
            if u in genre_map:
                genre = genre_map[u]
                break

            if ask_for_create("genre", i):
                genre = i
                break

        return genre


    def help_screen(self) -> None:
        print(HELP_MESSAGE)

    def shell(self) -> None:
        print(f"Welcome to the {PROGRAMM} shell!")
        print(f"Type '{BColors.OKBLUE}help{BColors.ENDC}' for a list of commands.")
        print("")

        while True:
            i = input("> ")
            if i == "help":
                self.help_screen()
            elif i == "genre":
                self.genre
            elif i == "exit":
                break
            else:
                print("Unknown command. Type 'help' for a list of commands.")

def main():
    parser = argparse.ArgumentParser(
        prog=PROGRAMM,
        description=DESCRIPTION,
        epilog='This is just a development cli. The real frontend is coming soon.'
    )
    parser.add_argument('--genre', '-g', action='store_const', required=False, help="choose a genre to download from")
    args = parser.parse_args()
