from ..utils import BColors
from ..utils.shared import get_random_message


def cli_function(function):
    def wrapper(*args, **kwargs):
        silent = kwargs.get("no_cli", False)
        if "no_cli" in kwargs:
            del kwargs["no_cli"]
        
        if silent:
            return function(*args, **kwargs)
            return
        
        code = 0
        
        print_cute_message()
        print()
        try:
            code = function(*args, **kwargs)
        except KeyboardInterrupt:
            print("\n\nRaise an issue if I fucked up:\nhttps://github.com/HeIIow2/music-downloader/issues")

        finally:
            print()
            print_cute_message()
            print("See you soon! :3")
        
        exit()
            
    return wrapper


def print_cute_message():
    message = get_random_message()
    try:
        print(message)
    except UnicodeEncodeError:
        message = str(c for c in message if 0 < ord(c) < 127)
        print(message)


def highlight_placeholder(text: str) -> str:
    return text.replace("<", f"{BColors.BOLD}<").replace(">", f">{BColors.ENDC}")


HELP_MESSAGE = highlight_placeholder(f"""{BColors.HEADER}To search:{BColors.ENDC}
> s: <query/url>
> s: https://musify.club/release/some-random-release-183028492
> s: #a <artist> #r <release> #t <track>

If you found the same object twice from different sources you can merge those objects.
Then it will use those sources. To do so, use the {BColors.BOLD}m{BColors.ENDC} command.

{BColors.HEADER}To download:{BColors.ENDC}
> d: <id/url>
> dm: 0, 3, 4   # merge all objects into one and download this object
> d: 1
> d: https://musify.club/release/some-random-release-183028492

{BColors.HEADER}To inspect an object:{BColors.ENDC}
If you inspect an object, you see its adjacent object. This means for example the releases of an artist, or the tracks of a release.
You can also merge objects with the {BColors.BOLD}m{BColors.ENDC} command here.

> g: <id/url>
> gm: 0, 3, 4   # merge all objects into one and inspect this object
> g: 1
> g: https://musify.club/release/some-random-release-183028492""")


class COMMANDS:
    AGREE = {"y", "yes", "ok"}
    DISAGREE = {"n", "no"}
    EXIT = {"exit"}
    HELP = {"help", "h"}


def ask_for_bool(msg: str) -> bool:
    i = input(f"{msg} ({BColors.OKGREEN.value}Y{BColors.ENDC.value}/{BColors.FAIL.value}N{BColors.ENDC.value})? ").lower()
    return i in COMMANDS.AGREE


def ask_for_create(name: str, value: str) -> bool:
    return ask_for_bool(f"Do you want to create the {name} {BColors.OKBLUE}{value}{BColors.ENDC}?") 
    